@@ -1,6 +1,6 @@
 /*========================================================================*\
 
-Copyright (c) 1990-2004  Paul Vojta
+Copyright (c) 1990-2014  Paul Vojta
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -167,6 +167,10 @@ in xdvi.c.
 #include <X11/Xfuncs.h>
 #include <X11/Intrinsic.h>
 
+#if HAVE_XI21
+# include <X11/extensions/XInput2.h>	/* for hi-res (smooth) scrolling */
+#endif
+
 #ifndef MAXPATHLEN
 #  ifdef PATH_MAX
 #    define MAXPATHLEN PATH_MAX
@@ -188,14 +192,6 @@ extern unsigned long time_start, time_end;
 
 #include <X11/Xmd.h>	/* get WORD64 and LONG64 */
 
-#ifndef WORD64
-#  ifdef LONG64
-typedef unsigned int xuint32;
-#  else
-typedef unsigned long xuint32;
-#  endif
-#endif
-
 #if defined(HAVE_STDINT_H)
 #include <stdint.h>
 #elif defined(HAVE_INTTYPES_H)
@@ -243,26 +239,9 @@ typedef unsigned long xuint32;
 
 #include "kpathsea/c-dir.h" /* dirent.h, NAMLEN */
 
-extern KPSEDLL char *kpathsea_version_string;
-
-#ifndef	NeedFunctionPrototypes
-#  if	__STDC__
-#    define   NeedFunctionPrototypes	1
-#  else
-#    define   NeedFunctionPrototypes	0
-#  endif
-#endif
-
-#ifndef	NeedWidePrototypes
-#  define NeedWidePrototypes	NeedFunctionPrototypes
-#endif
-
-#ifndef	NeedVarargsPrototypes
-#  define NeedVarargsPrototypes	NeedFunctionPrototypes
-#endif
-
-#include "kpathsea/c-vararg.h"
+#include <stdarg.h>
 #include "kpathsea/c-fopen.h"
+#include "kpathsea/version.h"
 
 #ifndef	_XFUNCPROTOBEGIN
 #  define _XFUNCPROTOBEGIN
@@ -318,13 +297,8 @@ extern KPSEDLL char *kpathsea_version_string;
 
 typedef	unsigned char	ubyte;
 
-/* #if NeedWidePrototypes */ /* bug #1570481 */
 typedef	unsigned int	wide_ubyte;
 typedef	int		wide_bool;
-/* #else */
-/* typedef	ubyte		wide_ubyte; */
-/* typedef	Boolean		wide_bool; */
-/* #endif */
 
 #if defined(MAKEPK) && !defined(MKTEXPK)
 #  define MKTEXPK 1
@@ -517,7 +491,10 @@ extern struct x_resources {
 #ifdef MOTIF
     char       *prefs_editor_list;
 #endif
-    Boolean	t1lib;
+    Boolean	freetype;
+#if HAVE_XI21
+    Boolean	xi2scrolling;
+#endif
     const char *src_pos;
     const char *find_string;
     const char *text_encoding;
@@ -526,8 +503,10 @@ extern struct x_resources {
     const char *file_history;
     int		file_history_size;
     Boolean	no_file_arg_use_history;
+#if RGB_ANTI_ALIASING
     subpixelOrderT subpixel_order;
     float	subpixel_energy[3];
+#endif
     Boolean	unique;
     Boolean	list_fonts;
     Boolean	reverse;
@@ -891,6 +870,17 @@ typedef void (*mouse_proc) (XEvent *);
 extern mouse_proc mouse_motion;
 extern mouse_proc mouse_release;
 
+struct mouse_acts {
+	Modifiers	mask;
+	Modifiers	value;
+	struct _LateBindings *late_bindings;
+	unsigned int	button;
+	struct xdvi_action *action;
+	struct mouse_acts *next;
+};
+
+extern	struct mouse_acts	*mouse_actions;
+
 #if GREY
 extern Pixel plane_masks[4];
 #endif
@@ -974,6 +964,51 @@ extern unsigned int color_list_max;	/* allocated size */
 extern Boolean color_warned;
 #endif /* COLOR */
 
+
+#if HAVE_XI21
+struct xi2_valinfo {
+	int		number;
+	double		increment;
+	double		lastval;
+	double		lastexact;
+	double		factor;
+	unsigned long	serial;
+};
+
+struct xi2_slave {
+	int		id;			/* slave device id */
+	unsigned int	flags;
+	unsigned char	enabled;
+	unsigned char	btn_mask;	/* buttons corresponding to valuators */
+	struct xi2_valinfo vert, horiz;
+	struct xi2_slave *next;
+};
+
+	/* flag bits */
+# define XI2_SLAVE_VERT			(1<<0)	/* if vert. valuator present */
+# define XI2_SLAVE_HORIZ		(1<<1)	/* if horiz. valuator present */
+
+struct xi2_master {
+	int		id;			/* master device id */
+	struct xi2_slave *slave;		/* currently active slave dev */
+	struct xi2_master *next;
+};
+
+
+extern	int		xi2_opcode;
+extern	Boolean		xi2_active;
+extern	struct xi2_master *xi2_masters;		/* linked list of master devs */
+extern	struct xi2_master *xi2_current;		/* current master device */
+extern	struct xi2_slave *xi2_slaves;		/* linked list of slave devs */
+
+extern	struct xi2_slave xi2_no_slave;		/* if no slave assigned yet */
+
+extern	void	xi2_init_valuators(struct xi2_slave *, XIAnyClassInfo **, int);
+extern	void	xi2_activate(void);
+
+#endif /* HAVE_XI21 */
+
+
 extern Boolean dvi_file_corrupted;
 
 extern short magnifier_stat;	/* 1 = wait for expose, -1 = destroy upon expose */
@@ -1185,7 +1220,9 @@ extern void motif_translations_hack(void);
 #define iconv libiconv
 #endif /* 0 */
 
+#if HAVE_ICONV_H
 typedef ICONV_CHAR_PPTR_TYPE iconv_char_pptrT;
+#endif /* HAVE_ICONV_H */
 
 #define DEVEL_MODE 0
 
